// IOMod.h: interface for the CIOMod class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IOMOD_H__30AAEC71_1DB0_11D3_8062_00105A1E5D53__INCLUDED_)
#define AFX_IOMOD_H__30AAEC71_1DB0_11D3_8062_00105A1E5D53__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


#include "AnalogIO.h"
#include "DigIO.h"
#include "DisplayIO.h"
#include "I2CMgr.h"
#include "Module.h"
#include "MyQue.h"


#undef DECL
#ifdef ENGINE_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif


const int MAX_IOMODULES = 20;

//#ifdef _WIN32_WCE
//prototype of thread proc for handling events from io devices
UINT EventThreadProc(void * pParam);

//this is a device class for handling events from modules
class DECL I2CEVENTDEVICE : public I2CDEVICE
{
public:
   I2CEVENTDEVICE(){};
   void Notify(const I2CMSG& Msg);
   MYQUE<I2CMSG> MsgQ;
};

//this is a device class I use for handling new device notifications.
//When this device sees a msg from the default address, an event is
//set so that the device list can be updated
class CIOModList;       //forward declare
class DECL NEWI2CDEVICE : public I2CDEVICE
{
public:
   NEWI2CDEVICE();
   virtual ~NEWI2CDEVICE();
   void Notify(const I2CMSG& Msg);
   HANDLE hNewDeviceEvent;
   bool Announced[128];       // Indicates a new announcement an address
};
//#endif

//This class is used by modules to handle events. If a module wants to
//handle events it should create one of these and set the handler for the
//IO Module, and then override the EventExec virtual function to do the
//event processing.
class DECL I2CEVENTHANDLER : public CObject
{
public:
   I2CEVENTHANDLER(CDigitalChannel Channel, TModule& mod);
   virtual ~I2CEVENTHANDLER(){};

   int m_Channel;
   TModule& m_Mod;
};

//This class is used to keep track of each IO module in the system.
//An object of this class is created for each module and added to a list

class DECL CIOMod : public CObject
{
   friend class CIOModList;
   friend class CIOModListDlg;

public:
   CIOMod(BYTE bAddr = 0x04, CIODeviceType type = DIGITAL);        //constructor
   virtual ~CIOMod();      //destructor

   //track device type
   CIODeviceType GetDeviceType(){return DeviceType;};
   void SetDeviceType(CIODeviceType Type){DeviceType = Type;};

   //for functions global to the device
   void Start();
   void Stop();

   CString GetDescription();

   //should be one or the other of these
   AnalogIO* GetAnalogPtr();
   DigitalIO* GetDigitalPtr();
   DisplayIO* GetDisplayPtr();

   //for handling events (alarms)
   void CallEventHandlers(const I2CMSG& Msg);      //called when an event msg comes in
   I2CEVENTDEVICE EventDevice;     //tracks channel and module to call for an event msg
   void SetEventHandler(I2CEVENTHANDLER& EventHandler);
   void ClearEventHandler(I2CEVENTHANDLER& EventHandler);

   virtual bool ShowProperties(CWnd * pParentWnd);

   //for getting at attributes.
   CString GetSerialNo() {return CString(serNo);};
   int GetModuleNumber() {return ModuleNumber;};

   HTREEITEM hItem;           // The handle to the entry for this module in the control panel.
   HANDLE hStopThread;
   HANDLE hThreadStopped;

protected:
   CIODeviceType DeviceType;  // The type of IO module, either analog or digital.
   BYTE Addr;                 // The i2c address assigned by xpert to the IO module.
   SERIALNO serNo;            // The serial number of the IO module.
   int ModuleNumber;          // The IO module number used to identify modules in a setup.

protected:
   IODevice* pIODevice;                    //ptr to either an AnalogIO or DigitalIO object
   CRITICAL_SECTION HandlerLock;
   TObArray<I2CEVENTHANDLER> HandlerList;  //list of event handlers to call if an event happens
};

//This class handles the list of IO module objects.  There is only
//one list for the application.  The list is part of the Engine
//where it may be accessed by other DLLs
class DECL CIOModList
{
   friend class CEngine;
   friend class CIOModListDlg;
public:
   bool GetIOAddrAsString(CIODeviceType type, int ModuleNumber, int Channel, CString& string);
   CIOModList();
   virtual ~CIOModList();
   void UpdateList(bool OnInit);             //updates list of devices
   int GetSize();                            //number of devices in list
   CIOMod& GetAt(int nIndex);                //retrieves a specific device
   void Release();                           //frees elements in list
   void Initialize();                        //called to populate the list

   CIOMod* GetIOMod(CIODeviceType Type, int ModuleNumber);

   AnalogIO* GetAnalogIO(int ModuleNumber);
   DigitalIO* GetDigitalIO(int ModuleNumber);
   DisplayIO* GetDisplayIO(int ModuleNumber);
   CString GetDescription(CIODeviceType Type, int ModuleNumber);

   int GetDigitalCount();                          // Number of Digital Devices
   int GetAnalogCount();                           // Number of Analog Devices
   int GetDisplayCount();

// ComboBox helper routines (note: the RC compiler doesn't support unicode combo box data)
   void SetModuleList(CDialog& dlg, int ID);
   void SetDigitalInputList(CDialog& dlg, int ID, int ModuleNumber, int InitialModuleNumber, int& Channel, int RelatedChannel1=0, int RelatedChannel2=0);
   void SetAnalogInputList(CDialog& dlg, int ID, int ModuleNumber, int InitialModuleNumber, int& Channel, int RelatedChannel1=0, int RelatedChannel2=0);

// Channel management routines
   void FindChannel(CIODeviceType DeviceType, int& ModuleNumber, int& Channel);
   bool IsChannelInUse(CIODeviceType DeviceType, int ModuleNumber, int nChannel);
   void SetChannelInUse(CIODeviceType DeviceType, int ModuleNumber, int nChannel);
   void ClearChannelInUse(CIODeviceType DeviceType, int ModuleNumber, int nChannel);
   void ClearAllChannelsInUse();

   // Determines if io module usage is valid considering assigned module numbers and setup.
   bool ValidateConfig();

   //enum for channel types
   enum CChannelType
   {
      INPUT, OUTPUT, HI_SENSITIVITY, SHAFT_ENCODER, //digital modules
         RMYOUNG, DIFF_PAIR, NEGATED_INPUT
   };     //analog modules

//#ifdef _WIN32_WCE
   NEWI2CDEVICE * pNewI2CDevice;   //for listening to new devices added to i2c bus
//#endif

   HANDLE hStopThread;
   HANDLE hThreadStopped;

private:
   int DigitalCount;     // Number of Digital boards
   int AnalogCount;      // Number of Analog boards
   int DisplayCount;     // Number of Displays

   void Stop();                                    //stops all i2c devices
   void Start();                                   //starts all i2c devices

   BYTE AnalogChannelUsage[MAX_IOMODULES][NUM_ANALOGCHANNELS];
   BYTE DigitalChannelUsage[MAX_IOMODULES][NUM_DIGITALCHANNELS];

   CRITICAL_SECTION ListLock;
   //we can keep track of ow many times someone wants to use a channel
   TObArray<CIOMod> List;                  //The list itself
};


void DECL DDX_CBIndexOneBased(CDataExchange* pDX, int nIDC, int& Index);




#endif // !defined(AFX_IOMOD_H__30AAEC71_1DB0_11D3_8062_00105A1E5D53__INCLUDED_)
